@@ -31,7 +31,7 @@ Arguments:
     filepath:
         Where to look for all the pickle files of processed data.
 
-    format: ('beta_value', 'm_value', 'meth', 'meth_df', 'noob_df')
+    format: ('beta_value', 'm_value', 'meth', 'meth_df', 'noob_df', 'beta_csv', 'sesame')
         This also allows processed.csv file data to be loaded.
         If you need meth and unmeth values, choose 'meth' and
         it will return a data_containers object with the 'meth' and 'unmeth' values,
@@ -84,6 +84,8 @@ Use cases and format:
     format = meth_df: (dataframe)
         you have processed CSV files in the path specified and want a dataframe returned
         take the data_containers object returned and run `methylcheck.container_to_pkl(containers, save=True)` function on it.
+    format = noob_df: (dataframe)
+        loads noob_meth_values.pkl and noob_unmeth_values.pkl and returns two dataframes in a list
     format = sesame:
         for reading csvs processed using R's sesame package. It has a different format (Probe_ID, ind_beta, ind_negs, ind_poob) per sample.
         Only those probes that pass the p-value cutoff will be included.
@@ -217,9 +219,9 @@ Use cases and format:
                         index_col=index_column,
                         # these params speed up reading
                         usecols=columns,
-                        dtype={'illumina_id': str, 'IlmnID':str, 'noob_meth':np.float16,
-                            'noob_unmeth':np.float16, 'meth':np.float16, 'unmeth':np.float16,
-                            'beta_value':np.float16, 'm_value':np.float16, 'poobah_pval':np.float16},
+                        dtype={'illumina_id': str, 'IlmnID':str, 'noob_meth':np.float32,
+                            'noob_unmeth':np.float32, 'meth':np.float32, 'unmeth':np.float32,
+                            'beta_value':np.float32, 'm_value':np.float32, 'poobah_pval':np.float32},
                         engine='c',
                         memory_map=True, # load all into memory at once for faster reading (less IO)
                         #na_filter=False, # disable to speed read, if not expecting NAs
@@ -741,7 +743,7 @@ def load_sesame(filepath='.',
     """ called within .load() for loading sesame-processed samples in csvs (optionally gzipped).
     returns a dataframe of betas, with failing probes filtered out (unless overridden). """
     SESAME_RGLOB_PATTERN = '*_R0[0-9]C0[0-9]*_calls.csv*'
-    SESAME_FILENAME_REGEX = '(.*_R0[0-9]C0[0-9]).*_calls\.csv(\.gz)?'
+    SESAME_FILENAME_REGEX = '(.*_R0[0-9]C0[0-9]).*_calls\\.csv(\\.gz)?'
 
     # files are Sentrix_ID ... manifest code ... _calls.csv.gz
     total_parts = list(Path(filepath).rglob(SESAME_RGLOB_PATTERN))
@@ -759,8 +761,8 @@ def load_sesame(filepath='.',
                 usecols=columns,
                 # compression='infer' by default; works for .gz files
                 dtype={index_column: str,
-                    beta_column: np.float16,
-                    poobah_column: np.float16
+                    beta_column: np.float32,
+                    poobah_column: np.float32
                     },
                 engine='c', #fastest
                 memory_map=True, # load all into memory at once for faster reading (less IO)
